#include "RHB_Custom.h"

ClassImp (RHB_Custom)

void  RHB_Custom::InitParams (void) 
{
        char tmp[32];
        char const *filename;
        double offset_def=25.;
        struct tm *tm;
        char file_modified_time[100];

        Quanta_count=0;
        Mean_charge=0.;
         
	FEventProcessorThread::InitParams ();      // call official function
						   // and add user specific part :
        this->ELEC_X_QDC = new FParam *[N_STRIPS]; // create new params
        this->ELEC_Y_QDC = new FParam *[N_STRIPS];

        this->tx = GetParam("ELEC_X_t");           // get a param already declared
        this->ty = GetParam("ELEC_Y_t");
        this->t0x = this->tx->GetValue();
        this->t0y = this->ty->GetValue();
        this->t1x = this->tx->GetValue();
        this->t1y = this->ty->GetValue();

        this->sum_x_inst = 0.0;
        this->sum_y_inst = 0.0;
        this->elecX=0;
        this->elecY=0;

        this->Time_ELEC_X    = new FParamBrutSimple ();
        this->Time_ELEC_X->SetName    ("Time_ELEC_X");
        this->AddRawParam (this->Time_ELEC_X);
        this->Time_ELEC_Y    = new FParamBrutSimple ();
        this->Time_ELEC_Y->SetName    ("Time_ELEC_Y");
        this->AddRawParam (this->Time_ELEC_Y);

        for(int i=0;i<N_STRIPS;i++)
        {
                sprintf(tmp, "ELEC_X_QDC%d",i+1);
                this->ELEC_X_QDC[i] = GetParam(tmp);
                sprintf(tmp, "ELEC_Y_QDC%d",i+1);
                this->ELEC_Y_QDC[i] = GetParam(tmp);
                this->q_X[i]=0.;
                this->q_Y[i]=0.;
                this->offXY[i][0]=0.;
                this->offXY[i][1]=0.;
        }

        this->charge_X_sum    = new FParamBrutSimple ();
        this->charge_Y_sum    = new FParamBrutSimple ();
        this->charge_X_sum->SetName    ("ELEC_X_SUM");
        this->charge_Y_sum->SetName    ("ELEC_Y_SUM");
        this->AddRawParam (this->charge_X_sum);
        this->AddRawParam (this->charge_Y_sum);

        FILE * offset_file;
        filename="background_noise_XY.txt"; // Measured backgound noise file
        offset_file=fopen(filename,"r");
        if(offset_file!=NULL)
        {
                struct stat fileStat;
                stat(filename,&fileStat);
                tm=localtime(&fileStat.st_mtime);
                sprintf(file_modified_time, "%02d/%02d/%04d à %02d:%02d:%02d",tm->tm_mday,tm->tm_mon+1,tm->tm_year+1900,tm->tm_hour,tm->tm_min,tm->tm_sec);
                printf("\n### Background noise file modified on %s ###\n",file_modified_time);
                for(int i=0;i<N_STRIPS;i++)
                {
                        int tmp;
                        double offX=1.;
                        double offY=1.;
                        if(fscanf(offset_file,"%d %lf %lf",&tmp,&offX,&offY)>0)
                        {
                                this->offXY[i][0]=(double)offX;
                                this->offXY[i][1]=(double)offY;
                        }
                }
                fclose(offset_file);
        }
        else printf("\n### No background file found\n default value used : %lf ###\n",offset_def);
        
        this->tcount = GetParam("myScaler_COUNT_t");  // get a param already declared
        this->Count = GetParam("myScaler_nb_quanta");
        this->t0c = this->tcount->GetValue();
        this->t1c = this->tcount->GetValue();

        this->Time_COUNT  = new FParamBrutSimple ();  // create new params
        this->Time_COUNT->SetName("Time_COUNT");
        this->AddRawParam (this->Time_COUNT);

	this->thvn = GetParam("HV_N_t");	      // get a param already declared
        this->thvp = GetParam("HV_P_t");	
	this->t0hvn = this->thvn->GetValue();	      // init intermediary parameters
        this->t0hvp = this->thvp->GetValue();
        this->t1hvn = this->thvn->GetValue();
        this->t1hvp = this->thvp->GetValue();

	this->Time_HV_N    = new FParamBrutSimple ();  // create new params
        this->Time_HV_N->SetName    ("Time_HV_N");
        this->AddRawParam (this->Time_HV_N);
        this->Time_HV_P    = new FParamBrutSimple ();
        this->Time_HV_P->SetName    ("Time_HV_P");
        this->AddRawParam (this->Time_HV_P);

}

void RHB_Custom::InitHistos (void)
{
        FEventProcessorThread::InitHistos ();         // call official function

        this->Map_inst=this->GetHisto("INST_DOSE");   // get a histo already declared
        this->Map_tot=this->GetHisto("TOTAL_DOSE");
}


void RHB_Custom::SetParamValues (void) 
{
        Double_t ele2pC_coef = 4.5777e-05;
        Double_t Offset = 25.0;
        Double_t StripSize = 3.0;
        Double_t Calib = 2.7E-2;

        Double_t val=0.;
        Double_t sum_t_inst=0.;
        Int_t both_electro=0;        

	FEventProcessorThread::SetParamValues ();     //  call official function

        if(this->tx->IsOn())
        {
                if(this->t0x<0) this->t0x=this->tx->GetValue();
                if(this->t1x<0) this->t1x=this->tx->GetValue();
                if(this->tx->GetValue()-this->t1x>=1.0e9)
                {
                        this->t1x=this->tx->GetValue();
                        this->Time_ELEC_X->SetValue((this->tx->GetValue()-this->t0x)/1E9);
                        this->charge_X_sum->SetValue(this->sum_x_inst/Calib);
                }
        }

        if(this->ty->IsOn())
        {
                if(this->t0y<0) this->t0y=this->ty->GetValue();
                if(this->t1y<0) this->t1y=this->ty->GetValue();
                if(this->ty->GetValue()-this->t1y>=1.0e9)
                {
                        this->t1y=this->ty->GetValue();
                        this->Time_ELEC_Y->SetValue((this->ty->GetValue()-this->t0y)/1E9);
                        this->charge_Y_sum->SetValue(this->sum_y_inst/Calib);
                }
        }

        for(int i=0;i<N_STRIPS;i++)
        {
                if(this->ELEC_X_QDC[i]->IsOn())
                {
                        Offset=offXY[i][0];
                        if(i==0) this->sum_x_inst=0.;
                        val=(this->ELEC_X_QDC[i]->GetValue()*ele2pC_coef-Offset);
                        if(val>=Offset*.001)
                        {
                                this->q_X[i]=val;
                                this->sum_x_inst+=val;
                        }
                        else this->q_X[i]=0.;
                        this->elecX=1;
                }
                if(this->ELEC_Y_QDC[i]->IsOn())
                {
                        Offset=offXY[i][1];
                        if(i==0) this->sum_y_inst=0.;
                        val=(this->ELEC_Y_QDC[i]->GetValue()*ele2pC_coef-Offset);
                        if(val>=Offset*.001)
                        {
                                this->q_Y[i]=val;
                                this->sum_y_inst+=val;
                        }
                        else this->q_Y[i]=0.;
                        this->elecY=1;
                }
        }
        both_electro=this->elecX+this->elecY;

        if(both_electro==2)
        {
                sum_t_inst=(this->sum_x_inst+this->sum_y_inst)/2.;
                for(int i=0;i<N_STRIPS;i++)
                {
                        for(int j=0;j<N_STRIPS;j++)
                        {
                                if(this->sum_x_inst>0.&&this->sum_y_inst>0.)
                                        val=this->q_X[i]/this->sum_x_inst*this->q_Y[j]/this->sum_y_inst*sum_t_inst;
                                else val=0.;
                                this->Map_inst->SetBinContent(i,j,val);
                                val+=this->Map_tot->GetBinContent(i,j);
                                this->Map_tot->SetBinContent(i,j,val);
                        }
                }
                this->elecX=0;
                this->elecY=0;
        }

        if(this->Count->IsOn()) Quanta_count=this->Count->GetValue();

        if(this->tcount->IsOn())
        {
                if(this->t0c<0) this->t0c=this->tcount->GetValue();
                if(this->t1c<0) this->t1c=this->tcount->GetValue();
                if(this->tcount->GetValue()-this->t1c>=1.0e9)
                {
                        this->t1c=this->tcount->GetValue();
                        this->Time_COUNT->SetValue((this->tcount->GetValue()-this->t0c)/1E9);
                        if(Quanta_count>0&&Mean_charge>0.)
                                cout << "Calibration ratio: " << Mean_charge/2. << "/"<<Quanta_count << "="<< Mean_charge/(2.*Quanta_count) << " pC/part" << endl;
                }
        }

	if(this->thvn->IsOn())
        {
                if(this->t0hvn<0) this->t0hvn=this->thvn->GetValue();
                if(this->t1hvn<0) this->t1hvn=this->thvn->GetValue();
                if(this->thvn->GetValue()-this->t1hvn>=1.0e9)
                {
                        this->t1hvn=this->thvn->GetValue();
                        this->Time_HV_N->SetValue((this->thvn->GetValue()-this->t0hvn)/1E9);
		}
	}

 	if(this->thvp->IsOn())
        {
                if(this->t0hvp<0) this->t0hvp=this->thvp->GetValue();
                if(this->t1hvp<0) this->t1hvp=this->thvp->GetValue();
                if(this->thvp->GetValue()-this->t1hvp>=1.0e9)
                {
                        this->t1hvp=this->thvp->GetValue();
                        this->Time_HV_P->SetValue((this->thvp->GetValue()-this->t0hvp)/1E9);
                }
        }

}


